# -*- coding: utf8 -*-
# demonstrate_dialogs
# This ProSEM script demonstrates examples of different types of wx dialogs callable from ProSEM/Python
# It doesn't do any actions other than print the results of the dialog

# There are many websites with helpful information about using dialogs from Wx widgets from Python. 
# for example:  https://www.blog.pythonlibrary.org/2010/06/26/the-dialogs-of-wxpython-part-1-of-2/



import sys
import os
import traceback
import datetime
import wx
    # Important: to use Wx elements, a Wx object must be instanced _prior_ to the ProSEM object instantiation.
    # See the line:          app=wx.App(False)
    # down in the main code block of this script. 

# Import classes from the ProSEMpy scripting interface:
from ProSEMpy import *

def process(prosem, file=sys.stdout):
    '''
    # show different types of Wx dialogs just for demonstration
    '''


    keep_going = True     # loop through the user choice until the user selects cancel
    while keep_going:

        #====================   SINGLE CHOICE INPUT: Choosing from a list of options   ===========================
        print ('Choice dialog where the user is given a list of options to choose from.')
        try:
            choices = ['Message Boxes, 1-Button','Message Boxes, Multi-Button','Text Input','Files']
            whichDialog = wx.GetSingleChoice("Which Dialog to Demonstrate? \n (Click Cancel to Exit)","Options",choices)
                # wx.GetSingleChoiceIndex might also be useful in some cases, returning the index number of the selection in the options list, -1 if cancelled.
            if whichDialog != "": 
                print("Chosen answer is: ",whichDialog)
            else:
                print ("The cancel button was clicked, so the choice is no choice.")
                keep_going = False
        except:
            print("     Error creating dialog. Perhaps wx library is not loaded?")
            traceback.print_exc()
    
        if whichDialog == 'Message Boxes, 1-Button':
   
            #====================   MESSAGE BOXES WITH JUST ONE BUTTON    ===========================
            #   Simple dialogs to show a bit of information, give a message or error
            # Button Options:
            #         wx.CANCEL   wx.YES_NO   wx.YES_DEFAULT  wx.NO_DEFAULT
            #  Style Options:
            #         wx.ICON_EXCLAMATION   wx.ICON_ERROR   wx.ICON_HAND    wx.ICON_INFORMATION     wx.ICON_QUESTION

            #   Information
            print ('Demonstrate an Information dialog with just an OK button.')
            try:
                messageText = 'This script demonstrates some of the many different types of dialogs available with the Wx library, which is included with ProSEM.'
                dialogAnswer = wx.MessageBox(messageText, "Box Title",  wx.OK | wx.ICON_INFORMATION)
            except:
                print("     Error creating dialog.")
                traceback.print_exc()

            #   OK Button with an Exclamation Mark icon!    typically used for warnings. 
            print ('Demonstrate a Warning dialog with just an OK button.')
            try:
                messageText = 'This script will not change any data, and even though File dialogs are demonstrated, no files are changed or created.'
                dialogAnswer = wx.MessageBox(messageText, 'Just so you know...',  wx.OK | wx.ICON_EXCLAMATION)
            except:
                print("     Error creating dialog.")
                traceback.print_exc()

            #   Error message dialog 
            print ('Demonstrate an Error dialog with just an OK button.')
            try:
                dialogAnswer = wx.MessageBox('This is what an error message looks like', "Computer says no",  wx.OK | wx.ICON_ERROR)
            except:
                print("     Error creating dialog.")
                traceback.print_exc()

        elif whichDialog == 'Message Boxes, Multi-Button':

            #====================   CHOICE MESSAGE BOXES WITH YES/NO or OK/CANCEL BUTTONS FOR USER CHOICE  ===========================
            # Button Options:
            #         wx.CANCEL   wx.YES_NO   wx.YES_DEFAULT  wx.NO_DEFAULT
            #  Style Options:
            #         wx.ICON_EXCLAMATION   wx.ICON_ERROR   wx.ICON_HAND    wx.ICON_INFORMATION     wx.ICON_QUESTION

            #   Warning Icon, with answer options: OK or Cancel 
            print ('Demonstrate a message box with two buttons, OK and Cancel.')
            try:
                dialogAnswer = wx.MessageBox('This box has buttons for OK and Cancel', 'Demonstration of Cancel Dialog' , wx.CANCEL | wx.ICON_HAND)
                if (dialogAnswer == wx.YES) or (dialogAnswer == wx.OK):
                    print('The OK button was clicked.')
                elif (dialogAnswer == wx.NO) or (dialogAnswer == wx.CANCEL):
                    print('The Cancel button was clicked.')
                else:
                    print("You shouldn't have gotten here, the dialog returned something expected.")
            except:
                print("     Error creating dialog.")
                traceback.print_exc()

            #   Question Icon, with answer options: Yes or No 
            print ('Demonstrate a message box with two buttons, Yes and No.')
            try:
                messageText = 'This box has buttons for Yes and No, typically used to confirm an operation before proceeding.'
                dialogAnswer = wx.MessageBox(messageText , "Confirmation Dialog", wx.YES_NO | wx.ICON_QUESTION)
                if (dialogAnswer == wx.YES) or (dialogAnswer == wx.OK):
                    print('The Yes button was clicked.')
                elif (dialogAnswer == wx.NO) or (dialogAnswer == wx.CANCEL):
                    print('The No button was clicked.')
                else:
                    print("You shouldn't have gotten here, the dialog returned something expected.")
            except:
                print("     Error creating dialog.")
                traceback.print_exc()


        elif whichDialog == 'Text Input':

            #====================   TEXT INPUT BOX  ===========================
            print ('Demonstrates the user entering text into a dialog.')
            try:
                entryDialog = wx.TextEntryDialog(None,"Tell me something" , "What do you want to say")
                #entryDialog.SetValue ("Default nothings")            #optional, sets initial text in the entry box
                if entryDialog.ShowModal() == wx.ID_OK:
                    print ("Dialog says: ", entryDialog.GetValue())
                else:
                    print ("Dialog says nothing, because Cancel was pressed.")
            except:
                print("     Error creating dialog.")
                traceback.print_exc()

        elif whichDialog == 'Files':

            #====================   GET FILENAME TO OPEN     ===========================
            #
            #   Example wx file dialog for opening an existing file. 
            #   The filetype selector is set with the wildcard parameter, which can be changed as needed
            #   The options specified will save the directory of the selected file, so the next time this is called, it starts in that directory. 
            #   Could add option to the style parameter:  wx.FD_MULTIPLE   which allows the user to select multiple files
            #
            print ('Demonstrate a dialog to get a file from the user, including a File Type selector for image files.')
            try:
                currentDirectory = os.getcwd()
            # Create open file dialog
                selectionWildcard_1 = "Images (*.jpg,*.jpeg,*.png,*.tif,*.bmp)|*.jpg;*.jpeg;*.png;*.tif;*.bmp|All Files (*,*)|*.*"
                openFileDialog = wx.FileDialog(None, message="Choose a File", defaultDir=currentDirectory, defaultFile="", wildcard=selectionWildcard_1,
                    style=wx.FD_OPEN | wx.FD_FILE_MUST_EXIST | wx.FD_CHANGE_DIR)
                if openFileDialog.ShowModal() == wx.ID_OK:
                    print("The file that was selected is: ", openFileDialog.GetPath())
                else:
                    print("Cancel pressed.")
                openFileDialog.Destroy()
            except:
                print("     Error creating file dialog.")
                traceback.print_exc()


            #====================   GET FILENAME TO SAVE     ===========================
            #
            #   Example wx file dialog to get a filename for saving (writing)
            #   The filetype selector is set with the wildcard parameter, which can be changed as needed
            #   The options specified will save the directory of the selected file, so the next time this is called, it starts in that directory. 
            #   The Overwrite_Prompt is optional, but generally expected behavior for most programs 

            print ('Demonstrate a filename dialog asking for a filename for a new or replacement file.')
            print (' (if the user chooses an existing file, the OS automatically asks if the existing file should be overwritten,')
            print ('  because the option wx.FD_OVERWRITE_PROMPT is included. This is typical behavior for most programs. )')
            try:
                currentDirectory = os.getcwd()       # use the current directory as the default location to save to
            # Create save file dialog
                selectionWildcard_2 = "Text (*.txt)|*.txt|CSV (*.csv)|*.csv|All Files (*,*)|*.*"
                saveFileDialog = wx.FileDialog(None, message="Create a File to Save Data to", defaultDir=currentDirectory, defaultFile="", wildcard=selectionWildcard_2,
                    style=wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT | wx.FD_CHANGE_DIR)
                if saveFileDialog.ShowModal() == wx.ID_OK:
                    print("The file entered is: ", saveFileDialog.GetPath())
                else:
                    print("Cancel pressed.")
                saveFileDialog.Destroy()
            except:
                print("     Error creating file dialog.")
                traceback.print_exc()
        else:
            print ('No choice was selected.')

    print("That is all.")
    return(False)





# When called as a program, load a project passed on the command line,
# process it, then save it back to the original file:
if __name__ == '__main__':
    import os
    import traceback
    pause = False
    error = 1
    try:
        app=wx.App(False)
        prosem = GProSEM()
        project = sys.argv[1]
        print("Loading {0!r}...".format(project), end=" ", file=sys.stderr)
        prosem.load_project(project)
        print("{0} images".format(len(prosem)), file=sys.stderr)
        os.chdir(prosem.base)
        if process(prosem):
            print("Saving {0!r}...".format(prosem.file), end=" ", file=sys.stderr)
            prosem.save_project()
            print("{0} images".format(len(prosem)), file=sys.stderr)
            pause = False
        error = 0
    except KeyboardInterrupt:
        pause = False
    except:
        traceback.print_exc()
    if pause:
        print("Press enter to continue...", file=sys.stderr)
        sys.stdin.read(1)
    sys.exit(error)
